;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; GNU GENERAL PUBLIC LICENSE ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Axtell-Epstein-Young bargaining in regular grid model
;;
;; Copyright (C) 2010 David Poza, Felix Villafánez, José.M. Galán, José.I. Santos
;; 
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 3
;; of the License, or (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
;;
;; Contact information:
;; Jose I. Santos 
;;  INSISOC & University of Burgos, Spain. 
;;   e-mail: jisantos@ubu.es
;; David Poza 
;;  INSISOC & University of Valladolid, Spain. 
;;   e-mail: djpoza@gmail.com


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; BREEDS AND GLOBAL VARIABLES ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

breed [agents agent]

agents-own 
[
  tag
  component
  intratype-memory
  intertype-memory
  neighbours
  intra-best-reply
  inter-best-reply
  explored?
  intra-component
  inter-component
]

globals
[
  ;; Payoffs
  low 
  medium 
  high 
  portions
  
  ;; Output/input files
  experiment-name
  
  intratype-regime
  intertype-regime
  stop-condition
  stop-run?
  number-of-black-intra-components
  number-of-orange-intra-components
  number-of-inter-components
  component-index
]

;;;;;;;;;;;;;;;;;;;
;;; MODEL SETUP ;;;
;;;;;;;;;;;;;;;;;;;

to setup
  ca
  let grid-width sqrt number-of-agents - 1
  resize-world 0 grid-width 0 grid-width
  ask patches [set pcolor 139]
  
  set low 30
  set medium 50
  set high 70
  set portions (list low medium high)
  
  set-default-shape agents "person"
  
  if layout = "random"
  [
      ask patches [sprout-agents 1 [set color black set tag "black" ]]
      ask n-of (number-of-agents / 2) agents [set color orange set tag "orange"]
  ]
  
  if layout = "2 stripes"
  [
      ask patches with [pxcor < grid-width / 2] [sprout-agents 1 [set color black set tag "black" ]]
      ask patches with [pxcor >= grid-width / 2] [sprout-agents 1 [set color orange set tag "orange"]]
  ]
  
  if layout = "4 squares"
  [
      ask patches with [pxcor < grid-width / 2 and pycor >= grid-width / 2 ] [sprout-agents 1 [set color black set tag "black" ]]
      ask patches with [pxcor < grid-width / 2 and pycor < grid-width / 2 ] [sprout-agents 1 [set color orange set tag "orange"]]
      ask patches with [pxcor >= grid-width / 2 and pycor >= grid-width / 2 ] [sprout-agents 1 [set color orange set tag "orange"]]
      ask patches with [pxcor >= grid-width / 2 and pycor < grid-width / 2 ] [sprout-agents 1 [set color black set tag "black"]]
  ]
  
  if layout = "customized"
  [
      ask patches [sprout-agents 1 [set color black set tag "black"]]
      ask one-of agents [set color orange set tag "orange"]      
  ]
end

to setup-game
  reset-ticks
  ask patches [set pcolor 139]
  ask agents [ set intratype-memory n-values memory-size [one-of portions] 
    set intertype-memory n-values memory-size [one-of portions]]
  ask agents [set neighbours agents-on neighbors]
  ask agents [set intra-best-reply 0 set inter-best-reply 0]
  initialize-output-files
  initialize-game-components
  do-ploting
end

to initialize-game-components
    
  find-all-components-in ("black")
  find-all-components-in ("orange")
  find-all-components-between-tags
  
  set number-of-black-intra-components max [intra-component] of agents with [tag = "black"]
  set number-of-orange-intra-components max [intra-component] of agents with [tag = "orange"]
  set number-of-inter-components max [inter-component] of agents
  
  set intratype-regime []
  foreach n-values number-of-black-intra-components [? + 1 ] [  
    set intratype-regime fput (word "intra:black:" ?  ":none") intratype-regime ]
  foreach n-values number-of-orange-intra-components [? + 1 ] [  
    set intratype-regime fput (word "intra:orange:" ?  ":none") intratype-regime ]
  
  set intertype-regime []
  foreach n-values number-of-inter-components [? + 1 ] [  
    set intertype-regime fput (word "inter:black:" ?  ":none") intertype-regime ]
  foreach n-values number-of-inter-components [? + 1 ] [  
    set intertype-regime fput (word "inter:orange:" ?  ":none") intertype-regime ]

  set stop-run? false
  set stop-condition []

end


;;;;;;;;;;;;;;;;;;;;;;;
;;; MAIN PROCEDURES ;;;
;;;;;;;;;;;;;;;;;;;;;;;

to go

  ask agents 
    [
          
      let agent1 self
      let agent2 one-of neighbours
    
      
      ;; AGENT 1'S DECISION
      let agent1-decision 0
      
      ifelse random-float 1 < epsilon
      ;; random
        [
          set agent1-decision one-of portions
        ]
      ;; rational
        [
          if decision-rule = "Maximize the expected benefit"
            [
              ifelse [tag] of agent1 = [tag] of agent2 [set agent1-decision max-ben ([intratype-memory] of agent1)] [set agent1-decision max-ben ([intertype-memory] of agent1)]
            ]  
          if decision-rule = "Demand the best reply against the opponent's most frequent demand" 
            [
              ifelse [tag] of agent1 = [tag] of agent2 [set agent1-decision best-reply ([intratype-memory] of agent1)] [set agent1-decision best-reply ([intertype-memory] of agent1)]
            ]
        ]
                
     
      ;; AGENT 2'S DECISION
      let agent2-decision 0
      ifelse random-float 1 < epsilon
      ;; random
        [
          set agent2-decision one-of portions
        ]
      ;; rational
        [
          if decision-rule = "Maximize the expected benefit"
            [
                        ifelse [tag] of agent1 = [tag] of agent2 [set agent2-decision max-ben ([intratype-memory] of agent2)] [set agent2-decision max-ben ([intertype-memory] of agent2)]
            ]
          if decision-rule = "Demand the best reply against the opponent's most frequent demand"
            [
              ifelse [tag] of agent1 = [tag] of agent2 [set agent2-decision best-reply ([intratype-memory] of agent2)] [set agent2-decision best-reply ([intertype-memory] of agent2)]
            ]  
   
        
        ]
        
                
 
      ;; UPDATE THE AGENT'S MEMORIES   
      ifelse [tag] of agent1 = [tag] of agent2
        [
          ask agent1 [set intratype-memory fput agent2-decision intratype-memory set intratype-memory but-last intratype-memory]
          ask agent2 [set intratype-memory fput agent1-decision intratype-memory set intratype-memory but-last intratype-memory]
        ]
        [
          ask agent1 [set intertype-memory fput agent2-decision intertype-memory set intertype-memory but-last intertype-memory]
          ask agent2 [set intertype-memory fput agent1-decision intertype-memory set intertype-memory but-last intertype-memory]
        ]
        
    
      ;; UPDATE THE AGENTS' BEST REPLY
      
      if decision-rule = "Demand the best reply against the opponent's most frequent demand"
      [
        ifelse [tag] of agent1 = [tag] of agent2 
        ;; intratype
        [ 
          set intra-best-reply best-reply (intratype-memory)
        ]
        [ ;; intertype
          set inter-best-reply best-reply (intertype-memory)    
        ] 
      ]
      
      if decision-rule = "Maximize the expected benefit"
        [
          ifelse [tag] of agent1 = [tag] of agent2 
          ;; intratype
          [ 
            set intra-best-reply max-ben (intratype-memory)
          ]
          [ ;; intertype
            set inter-best-reply max-ben (intertype-memory)    
          ] 
        ]


      
                  
    ]
      
      ;; UPDATE THE WORLD VIEW
      ask patches [set pcolor 139]
      
      ask agents
          [
            ifelse world-view ="intratype"
            [ ;; intratype
              if intra-best-reply = low [ask patch-at 0 0 [set pcolor yellow]]
              if intra-best-reply = medium [ask patch-at 0 0 [set pcolor green]]
              if intra-best-reply = high [ask patch-at 0 0 [set pcolor red]]
            ]
            [ ;; intertype
              if inter-best-reply = low [ask patch-at 0 0 [set pcolor yellow]]
              if inter-best-reply = medium [ask patch-at 0 0 [set pcolor green]]
              if inter-best-reply = high [ask patch-at 0 0 [set pcolor red]] 
            ]
          ]
          
 
      tick   
       
      check-stop-conditions
      if stop-run? [
         if output-file? [
           write-final-result-file
           if stop-condition = "C3" [ export-network-and-world ] ]
         stop   
      ]
     do-ploting
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Customize the spatial distribution ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to convert-to-orange
    if mouse-down? [ ask one-of agents-on patch mouse-xcor mouse-ycor [ set color orange set tag "orange" ] ]
end
to convert-to-black
    if mouse-down? [ ask one-of agents-on patch mouse-xcor mouse-ycor [ set color black set tag "black" ] ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check stop conditions ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to check-stop-conditions
 
  set intratype-regime []
  check-intra-regime "black"
  check-intra-regime "orange"

  set intertype-regime []
  check-inter-regime
  
  if (empty? filter [member? "none" ?] intratype-regime) and (empty? filter [member? "none" ?] intertype-regime) 
  [  
     ifelse (all-equal-item? map [get-regime ?] filter [member? "black" ?] intratype-regime) and 
            (all-equal-item? map [get-regime ?] filter [member? "orange" ?] intratype-regime) and 
            (all-equal-item? map [get-regime ?] intertype-regime)
       [ set stop-condition "C1" ]
       [ set stop-condition "C2" ]
     set stop-run? true
  ]
  if ticks = number-of-rounds [ set stop-run? true set stop-condition "C3" ]
    
end

to-report get-regime [regime-list]  ;; ok
  if member? ":EQ" regime-list [ report "EQ" ]
  if member? ":FR" regime-list [ report "FR" ]
  if member? ":IH" regime-list [ report "IQ" ]
  if member? ":IL" regime-list [ report "IQ" ]
  if member? ":none" regime-list [ report "none" ]
end

to-report all-equal-item? [ one-list ]  ;; ok
  let equal? true
  let mirror item 0 one-list
  foreach one-list [ 
    if mirror != ? [ set equal? false ]
    set mirror ? ]
  report equal? 
end

to check-intra-regime [one-tag]
  let n-components 0
  ifelse one-tag = "black" 
  [ set n-components number-of-black-intra-components ] 
  [ set n-components number-of-orange-intra-components ]

  foreach n-values n-components [? + 1 ]
  [  ;; EQ
     let component-list agents with [tag = one-tag and intra-component = ?]
     ifelse all? component-list [length filter [? = medium] intratype-memory >= floor ( (1 - epsilon) * memory-size )]
     [ set intratype-regime fput (word "intra:" one-tag ":" ?  ":EQ")  intratype-regime ]
     [ ;; Fractious state
       ifelse all? component-list [length filter [? = medium] intratype-memory <= ceiling (epsilon * memory-size )]
       [ set intratype-regime fput (word "intra:" one-tag ":" ?  ":FR")  intratype-regime ]
       [ set intratype-regime fput (word "intra:" one-tag ":" ?  ":none") intratype-regime ]  ;; no regime reached  
     ]
  ]
  
end

to check-inter-regime
  let n-components number-of-inter-components

  foreach n-values n-components [? + 1 ]
  [  ;; EQ
     let component-list agents with [inter-component = ?]
     ifelse all? component-list [length filter [? = medium] intertype-memory >= floor ( (1 - epsilon) * memory-size )]
     [ set intertype-regime fput (word "inter:black:" ?  ":EQ")  intertype-regime
       set intertype-regime fput (word "inter:orange:" ?  ":EQ")  intertype-regime ]
     [ ;; Inequity 
       ifelse all? component-list [length filter [? = medium] intertype-memory <= ceiling (epsilon * memory-size )]
       [  ifelse ( all? component-list with [tag = "black"] [length filter [? = high] intertype-memory >= floor ( (1 - epsilon) * memory-size )]) and 
                 ( all? component-list with [tag = "orange"] [length filter [? = low] intertype-memory >= floor ( (1 - epsilon) * memory-size )])  
          [ set intertype-regime fput (word "inter:black:" ?  ":IH")  intertype-regime
            set intertype-regime fput (word "inter:orange:" ?  ":IL")  intertype-regime ]
          [ ifelse ( all? component-list with [tag = "black"] [length filter [? = low] intertype-memory >= floor ( (1 - epsilon) * memory-size )]) and 
                   ( all? component-list with [tag = "orange"] [length filter [? = high] intertype-memory >= floor ( (1 - epsilon) * memory-size )])  
            [ set intertype-regime fput (word "inter:black:" ?  ":IL")  intertype-regime
              set intertype-regime fput (word "inter:orange:" ?  ":IH")  intertype-regime ]
            [ set intertype-regime fput (word "inter:black:" ?  ":FR")  intertype-regime
              set intertype-regime fput (word "inter:orange:" ?  ":FR")  intertype-regime ]
          ] ]
       [  set intertype-regime fput (word "inter:black:" ?  ":none")  intertype-regime
          set intertype-regime fput (word "inter:orange:" ?  ":none")  intertype-regime ]

     ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Agents' decision rules ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to-report best-reply [mem]
  let opponents-most-likely-demand one-of modes mem
    
  if opponents-most-likely-demand = low [report high]
  if opponents-most-likely-demand = medium [report medium]
  if opponents-most-likely-demand = high [report low]
end


to-report max-ben [mem]
 
  let prob-opponent-demands-low length (filter [? = low] mem) / memory-size
  let prob-opponent-demands-medium length (filter [? = medium] mem) / memory-size
  ;;let prob-opponent-demands-high length (filter [? = 70] mem) / memory-size

  let reward-if-30 30
  let reward-if-50 50 * prob-opponent-demands-low + 50 * prob-opponent-demands-medium
  let reward-if-70 70 * prob-opponent-demands-low
  
  let rewards-list (list reward-if-30 reward-if-50 reward-if-70)
  let max-benefit max rewards-list 
  
  let pos position max-benefit rewards-list
  if pos = 0 [report low]
  if pos = 1 [report medium]
  if pos = 2 [report high]  

end

;;;;;;;;;;;;;;
;; Plotting ;;
;;;;;;;;;;;;;;

to do-ploting ;; ok ;;

  set-current-plot "intratype simplex"
  clear-plot
  set-current-plot-pen "border"
  plotxy 0 0
  plotxy 0.5 ((sqrt 3) / 2)
  plotxy 1 0
  plotxy 0 0
  ;set-plot-pen-mode 2
  ask agents with [ intra-component > 0] 
  [ let sort-memory get-sort-memory intratype-memory
    ifelse tag = "black" 
    [   set-current-plot-pen "black"
        plot-circle (item 1 sort-memory + (item 2 sort-memory) / 2) ((item 2 sort-memory) * (sqrt 3) / 2)]
    [   set-current-plot-pen "orange"
        plot-circle (item 1 sort-memory + (item 2 sort-memory) / 2) ((item 2 sort-memory) * (sqrt 3) / 2)]
  ]
     
  set-current-plot "intertype simplex"
  clear-plot
  set-current-plot-pen "border"
  plotxy 0 0
  plotxy 0.5 ((sqrt 3) / 2)
  plotxy 1 0
  plotxy 0 0
  ;set-plot-pen-mode 2
  ask agents with [ inter-component > 0] 
  [ let sort-memory get-sort-memory intertype-memory
    ifelse tag = "black" 
    [   set-current-plot-pen "black"
        plot-circle (item 1 sort-memory + (item 2 sort-memory) / 2) ((item 2 sort-memory) * (sqrt 3) / 2)]
    [   set-current-plot-pen "orange"
        plot-circle (item 1 sort-memory + (item 2 sort-memory) / 2) ((item 2 sort-memory) * (sqrt 3) / 2)]
  ]

end
  
to-report get-sort-memory [memory]
  let sort-memory []
  foreach portions [
    let demand ?
    set sort-memory fput ((sum map [ ifelse-value (demand = ?) [1] [0] ] memory) / memory-size) sort-memory ]
  report reverse sort-memory  
end

to plot-circle [x y]
  let r 0.015
  let step 15
  set-plot-pen-mode 1
  foreach n-values step [ (360 / step) * ? ]
    [ plotxy  (x + r * sin ?) (y + r * cos ?) ]
  ;plotxy (x - r) y
  ;plotxy x (y + r)
  ;plotxy (x + r) y
  ;plotxy x (y - r) 
  set-plot-pen-mode 2
end

to plot-square [x y]
  let r 0.1
  set-plot-pen-mode 0
  foreach [ 45 135 225 315  45]
    [ plotxy  (x + r * sin ?) (y + r * cos ?) ]
  set-plot-pen-mode 2
end

;;;;;;;;;;;;;;;;;;;;;;;;;
;; NETWORK PROCEDURES ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

to export-tag-network [one-tag]
  ask agents [ set explored? false ]
  file-open (word experiment-name "-" one-tag "-network.net")
  let player-list sort agents with [ tag = one-tag ]
  file-print (word "*Vertices " (length player-list))
  foreach ( n-values length player-list  [?] )[ 
    file-print ( word  (? + 1)  " " ([who] of item ? player-list)  " " ([tag] of item ? player-list)) ]
  file-print (word "*Edges") 
  foreach player-list [
     ask ? [
      set explored? true 
      ask agents-on neighbors [
       if (not explored?) and (tag = one-tag) [
         file-print (word ((position myself player-list ) + 1) " " ( (position self player-list) + 1) " 1" )]
      ]
    ]
  ]
  file-close
end

;; These procedures are an adaptation from: Wilensky, U. (2005). NetLogo Giant Component model. 
;; http://ccl.northwestern.edu/netlogo/models/GiantComponent. 
;; Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

to find-all-components-in [ one-tag ]
  ask agents  [ set explored? false set inter-component 0]
  set component-index 0
  loop
  [ ifelse any? agents with [ ( (not explored?) and (tag = one-tag) ) and (any? (agents-on neighbors) with [tag = [tag] of myself]) ]
      [ let start one-of sort agents with [ ( (not explored?) and (tag = one-tag) ) and (any? (agents-on neighbors) with [tag = [tag] of myself]) ]
        set component-index ( component-index + 1)
        ask start [ explore-within-tag ]  ]
      [ stop ]
  ]
end

to explore-within-tag
  if explored? [ stop ]
  set explored? true
  set intra-component component-index
  ask ( agents-on neighbors) with [tag = [tag] of myself] [ explore-within-tag ]
end

;; to find all the connected components in the network of dissimilar tags
to find-all-components-between-tags
  ask agents  [ set explored? false set inter-component 0 ]
  set component-index 0
  loop
  [ ifelse any? agents with [ (not explored?) and (any? (agents-on neighbors) with [tag != [tag] of myself]) ]
      [ let start one-of sort agents with [ (not explored?) and (any? (agents-on neighbors) with [tag != [tag] of myself]) ]
        set component-index ( component-index + 1)
        ask start [ explore-between-tags ]  ]
      [ stop ]
  ]
end

to explore-between-tags
  if explored? [ stop ]
  set explored? true
  set inter-component component-index
  ask ( agents-on neighbors) with [tag != [tag] of myself] [ explore-between-tags ]
end

;;;;;;;;;;;;;;;;;;
;; Output files ;;
;;;;;;;;;;;;;;;;;;

to initialize-output-files 
  set experiment-name (word (remove ":" date-and-time) "." random 100 )
end

to write-final-result-file
  file-open (word experiment-name "_result.txt")
  file-type (word "parameters " number-of-agents " " memory-size " " layout " " epsilon " " number-of-rounds " ")  
  file-type (word "results " ticks " " stop-condition) 
  file-type (word " " number-of-black-intra-components)
  foreach filter [ (member? "intra" ?) and (member? "black" ?)] intratype-regime [ file-type (word " " ?)] 
  file-type (word " " number-of-orange-intra-components)
  foreach filter [(member? "intra" ?) and (member? "orange" ?)] intratype-regime [ file-type (word " " ?)] 
  file-type (word " " number-of-inter-components)
  foreach intertype-regime [ file-type (word " " ?)] 
  file-print " " 
  file-close
end

to export-network-and-world
  export-tag-network "black"
  export-tag-network "orange"
  export-world (word experiment-name "_world")
end
@#$#@#$#@
GRAPHICS-WINDOW
297
405
557
686
-1
-1
25.0
1
14
1
1
1
0
1
1
1
0
9
0
9
0
0
1
ticks

CHOOSER
8
10
168
55
number-of-agents
number-of-agents
4 16 36 64 100 144 196
4

SLIDER
8
62
167
95
memory-size
memory-size
3
20
10
1
1
NIL
HORIZONTAL

CHOOSER
10
231
168
276
layout
layout
"random" "2 stripes" "4 squares" "customized"
0

BUTTON
19
284
155
317
SETUP LAYOUT
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
19
363
156
396
GO ONCE
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
19
403
155
436
GO
go
T
1
T
OBSERVER
NIL
G
NIL
NIL

SLIDER
8
191
168
224
number-of-rounds
number-of-rounds
1
20000
10000
1
1
NIL
HORIZONTAL

CHOOSER
8
141
166
186
decision-rule
decision-rule
"Maximize the expected benefit" "Demand the best reply against the opponent's most frequent demand"
1

SLIDER
8
104
166
137
epsilon
epsilon
0
.1
0.02
.01
1
NIL
HORIZONTAL

CHOOSER
169
522
290
567
world-view
world-view
"intratype" "intertype"
0

MONITOR
306
10
799
55
NIL
intratype-regime
17
1
11

MONITOR
292
352
800
397
NIL
intertype-regime
17
1
11

MONITOR
19
445
158
490
NIL
stop-condition
17
1
11

MONITOR
181
10
298
55
intratype games
(word \"black:\" number-of-black-intra-components \" orange:\" number-of-orange-intra-components)
0
1
11

MONITOR
181
351
287
396
intertype games
number-of-inter-components
0
1
11

PLOT
181
64
487
341
intratype simplex
NIL
NIL
-0.1
1.1
-0.1
1.0
false
false
PENS
"border" 1.0 0 -7500403 true
"black" 1.0 0 -16777216 true
"orange" 1.0 0 -955883 true

PLOT
495
64
799
341
intertype simplex
NIL
NIL
-0.1
1.1
-0.1
1.0
false
false
PENS
"border" 1.0 0 -7500403 true
"black" 1.0 0 -16777216 true
"orange" 1.0 0 -955883 true

BUTTON
178
405
289
438
draw agents
ifelse color? = \"black\" \n [ convert-to-black ]\n [ convert-to-orange ]
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

CHOOSER
180
445
287
490
color?
color?
"black" "orange"
1

BUTTON
19
322
156
355
SETUP GAME
setup-game
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

TEXTBOX
338
92
358
114
H
18
15.0
1

TEXTBOX
461
292
487
316
M
18
55.0
1

TEXTBOX
216
291
231
313
L
18
45.0
1

TEXTBOX
653
91
668
113
H
18
15.0
1

TEXTBOX
773
290
789
312
M
18
55.0
1

TEXTBOX
534
291
549
313
L
18
45.0
1

TEXTBOX
175
571
286
637
Legend:\n* Low (yellow)\n* Medium (green)\n* High (red)\n
12
0.0
1

SWITCH
20
497
158
530
output-file?
output-file?
1
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="e01" repetitions="10" runMetricsEveryStep="true">
    <setup>setup
no-display</setup>
    <go>go</go>
    <final>display</final>
    <timeLimit steps="10000"/>
    <exitCondition>stop-run?</exitCondition>
    <enumeratedValueSet variable="number-of-agents">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="decision-rule">
      <value value="&quot;Demand the best reply against the opponent's most frequent demand&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="layout">
      <value value="&quot;random&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="number-of-rounds">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="epsilon">
      <value value="0.01"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="memory-size">
      <value value="10"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
